@@ -40,6 +40,9 @@ class NedPaperConfigDialog {
 		GtkWidget *m_radio_box_A4;
 		GtkWidget *m_radio_box_A3;
 		GtkWidget *m_radio_box_B4;
+		GtkWidget *m_radio_box_Letter;
+		GtkWidget *m_radio_box_Legal;
+		GtkWidget *m_radio_box_230x293;
 		GtkWidget *m_radio_box_portrait;
 		GtkWidget *m_radio_box_landspace;
 };
