@@ -23,7 +23,6 @@
 #include "pangocairotext.h"
 #include "mainwindow.h"
 #include "localization.h"
-#include "resource.h"
 
 #define CLEF_SHOW_WINDOW_WIDTH 90
 #define CLEF_SHOW_WINDOW_HEIGHT 100
@@ -39,10 +38,10 @@ _("F sharp Major; d sharp minor")
 };
 
 
-NedStaffContextDialog::NedStaffContextDialog(GtkWindow *parent, NedMainWindow *main_window,  bool allow_delete_system, int staff_count, int staff_pos, char *staff_name, char *staff_short_name, char *group_name, char* group_short_name, int clef_number, int octave_shift, int key_signature_number, int numerator,  int demoninator, unsigned int symbol, int volume, int midi_program, int channel, int tempo, int pan, int chorus, int play_transposed, int reverb, bool muted) :
-m_octave_shift(octave_shift), m_tempo(tempo), m_pan(pan), m_chorus(chorus), m_play_transposed(play_transposed), m_reverb(reverb), m_volume(volume), m_channel_value(channel),
+NedStaffContextDialog::NedStaffContextDialog(GtkWindow *parent, NedMainWindow *main_window, bool different_voices, bool allow_delete_system, int staff_count, int staff_pos, char *staff_name, char *staff_short_name, char *group_name, char* group_short_name, int clef_number, int octave_shift, int key_signature_number, int numerator,  int demoninator, unsigned int symbol, int volume[VOICE_COUNT], int midi_program[VOICE_COUNT], int channel, int tempo, int pan[VOICE_COUNT], int chorus, int play_transposed, int reverb, bool muted) :
+m_different_voices(different_voices), m_octave_shift(octave_shift), m_tempo(tempo), m_chorus(chorus), m_play_transposed(play_transposed), m_reverb(reverb), m_channel_value(channel),
 m_state(false), m_numerator_value(numerator), m_denominator_value(demoninator), m_symbol(symbol),
-m_key_signature_number(key_signature_number), m_current_clef(clef_number), m_midi_program_number(midi_program), m_config_changed(false),
+m_key_signature_number(key_signature_number), m_current_clef(clef_number),  m_config_changed(false),
 m_delete_staff(false), m_staff_pos(staff_pos), 
 m_staff_count(staff_count), m_staff_name(staff_name), m_staff_short_name(staff_short_name), m_group_name(group_name), m_group_short_name(group_short_name), m_muted(muted)  {
 	GtkWidget *dialog;
@@ -82,8 +81,11 @@ m_staff_count(staff_count), m_staff_name(staff_name), m_staff_short_name(staff_s
 	GtkWidget *channel_frame;
 	GtkWidget *mute_frame;
 	GtkWidget *mute_channel_vbox;
+	GtkWidget *voices_frame;
+	GtkWidget *vrb_vbox;
+	GtkWidget *vrb_hbox1;
+	GtkWidget *vrb_hbox2;
 	GdkColor bgcolor;
-	GtkTreePath* instrument_tree_path;
 	GtkListStore *position_list_store;
 	GtkCellRenderer *position_renderer;
 	GtkTreeViewColumn *position_column;
@@ -99,6 +101,13 @@ m_staff_count(staff_count), m_staff_name(staff_name), m_staff_short_name(staff_s
 	GtkWidget *group_name_vbox;
 	int i;
 	char Str[128];
+
+	for (i = 0; i < VOICE_COUNT; i++) {
+		m_midi_program_number[i] = midi_program[i];
+		m_pan[i] = pan[i];
+		m_volume[i] = volume[i];
+	}
+		
 	m_newclef = m_current_clef;
 
 	dialog = gtk_dialog_new_with_buttons(_("Staff context"), parent, (GtkDialogFlags) (GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT),
@@ -258,24 +267,12 @@ m_staff_count(staff_count), m_staff_name(staff_name), m_staff_short_name(staff_s
 	gtk_container_add (GTK_CONTAINER(group_short_name_frame), m_groupshortname);
 
 
-	volume_frame = gtk_frame_new(_("volume"));
-	m_volume_scale = gtk_vscale_new_with_range(0.0, 127.0, 1.0);
-	gtk_range_set_inverted(GTK_RANGE(m_volume_scale), TRUE);
-	gtk_range_set_value(GTK_RANGE(m_volume_scale), m_volume);
-	gtk_container_add (GTK_CONTAINER(volume_frame), m_volume_scale);
 
 	tempo_frame = gtk_frame_new(_("tempo"));
 	m_tempo_scale = gtk_vscale_new_with_range(20.0, 300.0, 1.0);
 	gtk_range_set_inverted(GTK_RANGE(m_tempo_scale), TRUE);
 	gtk_range_set_value(GTK_RANGE(m_tempo_scale), m_tempo);
 	gtk_container_add (GTK_CONTAINER(tempo_frame), m_tempo_scale);
-
-	pan_frame = gtk_frame_new(_("pan"));
-	m_pan_scale = gtk_vscale_new_with_range(20.0, 127.0, 1.0);
-	gtk_range_set_inverted(GTK_RANGE(m_pan_scale), TRUE);
-	gtk_range_set_value(GTK_RANGE(m_pan_scale), m_pan);
-	gtk_container_add (GTK_CONTAINER(pan_frame), m_pan_scale);
-
 	chorus_frame = gtk_frame_new(_("chorus"));
 	m_chorus_scale = gtk_vscale_new_with_range(20.0, 127.0, 1.0);
 	gtk_range_set_inverted(GTK_RANGE(m_chorus_scale), TRUE);
@@ -294,6 +291,18 @@ m_staff_count(staff_count), m_staff_name(staff_name), m_staff_short_name(staff_s
 	gtk_range_set_value(GTK_RANGE(m_reverb_scale), m_reverb);
 	gtk_container_add (GTK_CONTAINER(reverb_frame), m_reverb_scale);
 
+	volume_frame = gtk_frame_new(_("volume"));
+	// see also NedStaffContextDialog::SetCurrentVoiceParamsTo
+	m_volume_scale = gtk_vscale_new_with_range(0.0, 127.0, 1.0);
+	gtk_range_set_inverted(GTK_RANGE(m_volume_scale), TRUE);
+	gtk_container_add (GTK_CONTAINER(volume_frame), m_volume_scale);
+
+	pan_frame = gtk_frame_new(_("pan"));
+	m_pan_scale = gtk_vscale_new_with_range(20.0, 127.0, 1.0);
+	gtk_range_set_inverted(GTK_RANGE(m_pan_scale), TRUE);
+	// see also NedStaffContextDialog::SetCurrentVoiceParamsTo
+	gtk_container_add (GTK_CONTAINER(pan_frame), m_pan_scale);
+
 	instrumend_list_store = gtk_list_store_new (2, G_TYPE_INT, G_TYPE_STRING);
 
 	for (i = 0; i < NedResource::getNumInstruments(); i++) {
@@ -310,11 +319,8 @@ m_staff_count(staff_count), m_staff_name(staff_name), m_staff_short_name(staff_s
 	instrument_scroll = gtk_vscrollbar_new(GTK_ADJUSTMENT(m_instrument_adjustment));
 	gtk_tree_view_set_vadjustment (GTK_TREE_VIEW (m_instrument_list), GTK_ADJUSTMENT(m_instrument_adjustment));
 	gtk_widget_set_size_request(m_instrument_list, 150, 100);
-	sprintf(Str, "%d", m_midi_program_number);
-	instrument_tree_path = gtk_tree_path_new_from_string(Str);
-	gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW (m_instrument_list), instrument_tree_path, NULL, FALSE, 0.0, 0.0);
-	gtk_tree_view_set_cursor(GTK_TREE_VIEW (m_instrument_list), instrument_tree_path, NULL, FALSE);
 	g_signal_connect(dialog, "scroll-event", G_CALLBACK (OnScroll), (void *) this);
+	// see also NedStaffContextDialog::SetCurrentVoiceParamsTo
 
 	channel_frame = gtk_frame_new(_("channel"));
 	m_channel_spin_box = gtk_spin_button_new_with_range (1.0, 16.0, 1.0);
@@ -322,13 +328,55 @@ m_staff_count(staff_count), m_staff_name(staff_name), m_staff_short_name(staff_s
 	gtk_container_add (GTK_CONTAINER(channel_frame), m_channel_spin_box);
 
 	mute_frame = gtk_frame_new(_("muting"));
-	m_mute_box = gtk_check_button_new();
-	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m_mute_box), m_muted);
-	gtk_container_add (GTK_CONTAINER(mute_frame), m_mute_box);
+        m_mute_box = gtk_check_button_new();
+        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m_mute_box), m_muted);
+        gtk_container_add (GTK_CONTAINER(mute_frame), m_mute_box);
+
+
+	voices_frame = gtk_frame_new(_("voices"));
+	m_radio_all = gtk_radio_button_new_with_label(NULL, _("all"));
+	m_radio_1 = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON (m_radio_all), "1");
+	m_radio_2 = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON (m_radio_all), "2");
+	m_radio_3 = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON (m_radio_all), "3");
+	m_radio_4 = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON (m_radio_all), "4");
+
+
+	if (different_voices) {
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m_radio_1), TRUE);
+		SetCurrentVoiceParamsTo(0);
+		m_current_voice = 0;
+	}
+	else {
+		SetCurrentVoiceParamsTo(-1);
+		m_current_voice = -1;
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m_radio_all), TRUE);
+	}
+	g_signal_connect(m_radio_all, "toggled", G_CALLBACK (OnVoiceToggle), (void *) this);
+	g_signal_connect(m_radio_1, "toggled", G_CALLBACK (OnVoiceToggle), (void *) this);
+	g_signal_connect(m_radio_2, "toggled", G_CALLBACK (OnVoiceToggle), (void *) this);
+	g_signal_connect(m_radio_3, "toggled", G_CALLBACK (OnVoiceToggle), (void *) this);
+	g_signal_connect(m_radio_4, "toggled", G_CALLBACK (OnVoiceToggle), (void *) this);
+
+	vrb_vbox = gtk_vbox_new (FALSE, 0);
+	vrb_hbox1 = gtk_hbox_new (FALSE, 0);
+	vrb_hbox2 = gtk_hbox_new (FALSE, 0);
+
+	gtk_box_pack_start (GTK_BOX (vrb_vbox), m_radio_all, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (vrb_hbox1), m_radio_1, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (vrb_hbox1), m_radio_2, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (vrb_hbox2), m_radio_3, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (vrb_hbox2), m_radio_4, FALSE, FALSE, 0);
+
+	gtk_box_pack_start (GTK_BOX (vrb_vbox), vrb_hbox1, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (vrb_vbox), vrb_hbox2, FALSE, FALSE, 0);
+	gtk_container_add(GTK_CONTAINER(voices_frame), vrb_vbox);
+
+
 
 	mute_channel_vbox = gtk_vbox_new(FALSE, 2);
 	gtk_box_pack_start(GTK_BOX(mute_channel_vbox), channel_frame,  FALSE, FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(mute_channel_vbox), mute_frame, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(mute_channel_vbox), voices_frame, FALSE, FALSE, 0);
 
 	if (staff_count > 1) {
 
@@ -385,12 +433,12 @@ m_staff_count(staff_count), m_staff_name(staff_name), m_staff_short_name(staff_s
 	gtk_box_pack_start(GTK_BOX(score_hbox), key_frame, FALSE, FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(score_hbox), signature_name_vbox, FALSE, FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(score_hbox), group_name_vbox, FALSE, FALSE, 0);
-	gtk_box_pack_start(GTK_BOX(midi_hbox), volume_frame, FALSE, TRUE, 0);
 	gtk_box_pack_start(GTK_BOX(midi_hbox), tempo_frame, FALSE, TRUE, 0);
-	gtk_box_pack_start(GTK_BOX(midi_hbox), pan_frame, FALSE, TRUE, 0);
 	gtk_box_pack_start(GTK_BOX(midi_hbox), chorus_frame, FALSE, TRUE, 0);
 	gtk_box_pack_start(GTK_BOX(midi_hbox), reverb_frame, FALSE, TRUE, 0);
 	gtk_box_pack_start(GTK_BOX(midi_hbox), play_transposed_frame, FALSE, TRUE, 0);
+	gtk_box_pack_start(GTK_BOX(midi_hbox), volume_frame, FALSE, TRUE, 0);
+	gtk_box_pack_start(GTK_BOX(midi_hbox), pan_frame, FALSE, TRUE, 0);
 	gtk_box_pack_start(GTK_BOX(midi_hbox), m_instrument_list, TRUE, TRUE, 0);
 	gtk_box_pack_start(GTK_BOX(midi_hbox), instrument_scroll, FALSE, TRUE, 0);
 	gtk_box_pack_start(GTK_BOX(midi_hbox), mute_channel_vbox, FALSE, TRUE, 0);
@@ -416,17 +464,78 @@ m_staff_count(staff_count), m_staff_name(staff_name), m_staff_short_name(staff_s
 
 }
 
+void NedStaffContextDialog::SetCurrentVoiceParamsTo(int voice) {
+	int idx = 0;
+	char Str[128];
+	GtkTreePath* instrument_tree_path;
+	switch (voice) {
+		case -1: 
+		case 0: idx = 0; break;
+		case 1: idx = 1; break;
+		case 2: idx = 2; break;
+		case 3: idx = 3; break;
+		default: NedResource::Abort("NedStaffContextDialog::SetCurrentVoiceParamsTo");
+	}
+	gtk_range_set_value(GTK_RANGE(m_volume_scale), m_volume[idx]);
+	gtk_range_set_value(GTK_RANGE(m_pan_scale), m_pan[idx]);
+	sprintf(Str, "%d", m_midi_program_number[idx]);
+	instrument_tree_path = gtk_tree_path_new_from_string(Str);
+	gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW (m_instrument_list), instrument_tree_path, NULL, FALSE, 0.0, 0.0);
+	gtk_tree_view_set_cursor(GTK_TREE_VIEW (m_instrument_list), instrument_tree_path, NULL, FALSE);
+}
+
+void NedStaffContextDialog::GetCurrentVoiceParamsFrom(int voice, int volume[VOICE_COUNT], int pan[VOICE_COUNT], int instrument[VOICE_COUNT], bool *changed) {
+	int idx = 0;
+	GtkTreeSelection* selection;
+	GList *selected_rows;
+	int newvolume, newpan, newpgm;
+	switch (voice) {
+		case -1: 
+		case 0: idx = 0; break;
+		case 1: idx = 1; break;
+		case 2: idx = 2; break;
+		case 3: idx = 3; break;
+		default: NedResource::Abort("NedStaffContextDialog::GetCurrentVoiceParamsFRom");
+	}
+	newvolume = (int) gtk_range_get_value(GTK_RANGE(m_volume_scale));
+	if (newvolume != volume[idx]) {
+		volume[idx] = newvolume;
+		*changed = TRUE;
+	}
+	newpan = (int) gtk_range_get_value(GTK_RANGE(m_pan_scale));
+	if (newpan != pan[idx]) {
+		pan[idx] = newpan;
+		*changed = TRUE;
+	}
+	selection =  gtk_tree_view_get_selection (GTK_TREE_VIEW(m_instrument_list));
+	selected_rows = gtk_tree_selection_get_selected_rows (selection, NULL);
+	if ( g_list_length(selected_rows) > 0) {
+		GtkTreePath *path = (GtkTreePath *) g_list_first(selected_rows)->data;
+		char *tp = gtk_tree_path_to_string(path);
+		if (sscanf(tp, "%d", &(newpgm)) != 1) {
+			NedResource::Abort("NedStaffContextDialog::OnClose: error reading tree path(1)");
+		}
+	}
+	else {
+		newpgm = -1;
+	}
+	if (instrument[idx] != newpgm) {
+		instrument[idx] = newpgm;
+		*changed = true;
+	}
+}
+
 void NedStaffContextDialog::OnClose(GtkDialog *dialog, gint result, gpointer data) {
 	int i;
 	bool newmute;
-	GtkTreeSelection* selection;
-	GList *selected_rows;
-	int newchannel, newvolume, newpgm;
+	int newchannel;
 	int newnum, newdenom, newkeysig, newtempo;
-	int newpan, newchorus, newplaytransposed, newreverb, new_octave_shift;
+	int newchorus, newplaytransposed, newreverb, new_octave_shift;
 	const char *newname, *newshortname;
 	const char *newgroupname, *newshortgroupname;
-	bool diff;
+	bool diff, newdifferentvoices;
+	GtkTreeSelection* selection;
+	GList *selected_rows;
 
 	NedStaffContextDialog *context_dialog = (NedStaffContextDialog *) data;
 	context_dialog->m_delete_staff = FALSE;
@@ -473,21 +582,11 @@ void NedStaffContextDialog::OnClose(GtkDialog *dialog, gint result, gpointer dat
 		context_dialog->m_denominator_value = newdenom;
 		context_dialog->m_config_changed = TRUE;
 	}
-	newvolume = (int) gtk_range_get_value(GTK_RANGE(context_dialog->m_volume_scale));
-	if (newvolume != context_dialog->m_volume) {
-		context_dialog->m_volume = newvolume;
-		context_dialog->m_config_changed = TRUE;
-	}
 	newtempo = (int) gtk_range_get_value(GTK_RANGE(context_dialog->m_tempo_scale));
 	if (newtempo != context_dialog->m_tempo) {
 		context_dialog->m_tempo = newtempo;
 		context_dialog->m_config_changed = TRUE;
 	}
-	newpan = (int) gtk_range_get_value(GTK_RANGE(context_dialog->m_pan_scale));
-	if (newpan != context_dialog->m_pan) {
-		context_dialog->m_pan = newpan;
-		context_dialog->m_config_changed = TRUE;
-	}
 	newchorus = (int) gtk_range_get_value(GTK_RANGE(context_dialog->m_chorus_scale));
 	if (newchorus != context_dialog->m_chorus) {
 		context_dialog->m_chorus = newchorus;
@@ -575,18 +674,6 @@ void NedStaffContextDialog::OnClose(GtkDialog *dialog, gint result, gpointer dat
 	else {
 		context_dialog->m_group_short_name = NULL;
 	}
-	selection =  gtk_tree_view_get_selection (GTK_TREE_VIEW(context_dialog->m_instrument_list));
-	selected_rows = gtk_tree_selection_get_selected_rows (selection, NULL);
-	if ( g_list_length(selected_rows) > 0) {
-		GtkTreePath *path = (GtkTreePath *) g_list_first(selected_rows)->data;
-		char *tp = gtk_tree_path_to_string(path);
-		if (sscanf(tp, "%d", &newpgm) != 1) {
-			NedResource::Abort("NedStaffContextDialog::OnClose: error reading tree path(1)");
-		}
-	}
-	else {
-		newpgm = -1;
-	}
 	if (context_dialog->m_staff_count > 1) {
 		selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(context_dialog->m_position_list));
 		selected_rows = gtk_tree_selection_get_selected_rows (selection, NULL);
@@ -598,10 +685,6 @@ void NedStaffContextDialog::OnClose(GtkDialog *dialog, gint result, gpointer dat
 			}
 		}
 	}
-	if (context_dialog->m_midi_program_number != newpgm) {
-		context_dialog->m_midi_program_number = newpgm;
-		context_dialog->m_config_changed = true;
-	}
 	new_octave_shift = (int) gtk_spin_button_get_value(GTK_SPIN_BUTTON(context_dialog->m_octave_shift_spin_box));
 	switch (new_octave_shift) {
 		case 8: new_octave_shift = 12; break;
@@ -629,10 +712,19 @@ void NedStaffContextDialog::OnClose(GtkDialog *dialog, gint result, gpointer dat
 	else {
 		context_dialog->m_symbol = TIME_SYMBOL_NONE;
 	}
+	newdifferentvoices = !gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(context_dialog->m_radio_all));
+	if (newdifferentvoices != context_dialog->m_different_voices) {
+		context_dialog->m_config_changed = TRUE;
+		context_dialog->m_different_voices = newdifferentvoices;
+	}
+	context_dialog->GetCurrentVoiceParamsFrom(context_dialog->m_current_voice, context_dialog->m_volume, context_dialog->m_pan, context_dialog->m_midi_program_number,
+					 &(context_dialog->m_config_changed));
 	gtk_widget_destroy (GTK_WIDGET(dialog));
 }
 
-void NedStaffContextDialog::getValues(bool *state, bool *delete_staff, bool *delete_system, int *staff_pos, char **staff_name, char **staff_short_name, char **group_name, char **group_short_name, int *clef_number, int *octave_shift, int *key_signature_number, bool *ajdust_notes, int *numerator, int *demoninator,  unsigned int *symbol, int *volume, int *midi_program, int *channel, int *tempo, int *pan, int *chorus, int *play_transposed, int *reverb, bool *config_changed, bool *muted) {
+void NedStaffContextDialog::getValues(bool *state, bool *different_voices, bool *delete_staff, bool *delete_system, int *staff_pos, char **staff_name, char **staff_short_name, char **group_name, char **group_short_name, int *clef_number, int *octave_shift, int *key_signature_number, bool *ajdust_notes, int *numerator, int *demoninator,  unsigned int *symbol, int volume[VOICE_COUNT], int midi_program[VOICE_COUNT], int *channel, int *tempo, int pan[VOICE_COUNT], int *chorus, int *play_transposed, int *reverb, bool *config_changed, bool *muted) {
+	int i;
+	*different_voices = m_different_voices;
 	*delete_staff = m_delete_staff;
 	*delete_system = m_delete_system;
 	*state = m_state;
@@ -650,17 +742,19 @@ void NedStaffContextDialog::getValues(bool *state, bool *delete_staff, bool *del
 	*key_signature_number = m_key_signature_number;
 	*numerator = m_numerator_value;
 	*demoninator = m_denominator_value;
-	*volume = m_volume;
 	*tempo = m_tempo;
-	*pan = m_pan;
 	*chorus = m_chorus;
 	*play_transposed = m_play_transposed;
 	*reverb = m_reverb;
 	*octave_shift = m_octave_shift;
-	*midi_program = m_midi_program_number;
 	*channel = m_channel_value;
 	*muted = m_muted;
 	*symbol = m_symbol;
+	for (i = 0; i < VOICE_COUNT; i++) {
+		volume[i] = m_volume[i];
+		midi_program[i] = m_midi_program_number[i];
+		pan[i] = m_pan[i];
+	}
 	if (m_state) {
 		*config_changed = m_config_changed;
 	}
@@ -876,3 +970,58 @@ void NedStaffContextDialog::OnCutButtonToggled(GtkToggleButton *togglebutton, gp
 	}
 }
 
+void NedStaffContextDialog::OnVoiceToggle(GtkToggleButton *togglebutton, gpointer data) {
+	NedStaffContextDialog *context_dialog = (NedStaffContextDialog *) data;
+	int oldvoice = context_dialog->m_current_voice;
+
+	if (togglebutton == GTK_TOGGLE_BUTTON(context_dialog->m_radio_all)) {
+		if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(context_dialog->m_radio_all))) {
+			context_dialog->SetCurrentVoiceParamsTo(-1);
+			context_dialog->m_current_voice = -1;
+		}
+		else {
+			context_dialog->GetCurrentVoiceParamsFrom(-1, context_dialog->m_volume, context_dialog->m_pan,
+							context_dialog->m_midi_program_number, &(context_dialog->m_config_changed));
+		}
+	}
+	else if (togglebutton == GTK_TOGGLE_BUTTON(context_dialog->m_radio_1)) {
+		if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(context_dialog->m_radio_1))) {
+			context_dialog->SetCurrentVoiceParamsTo(0);
+			context_dialog->m_current_voice = 0;
+		}
+		else {
+			context_dialog->GetCurrentVoiceParamsFrom(0, context_dialog->m_volume, context_dialog->m_pan,
+							context_dialog->m_midi_program_number, &(context_dialog->m_config_changed));
+		}
+	}
+	else if (togglebutton == GTK_TOGGLE_BUTTON(context_dialog->m_radio_2)) {
+		if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(context_dialog->m_radio_2))) {
+			context_dialog->SetCurrentVoiceParamsTo(1);
+			context_dialog->m_current_voice = 1;
+		}
+		else {
+			context_dialog->GetCurrentVoiceParamsFrom(1, context_dialog->m_volume, context_dialog->m_pan,
+							context_dialog->m_midi_program_number, &(context_dialog->m_config_changed));
+		}
+	}
+	else if (togglebutton == GTK_TOGGLE_BUTTON(context_dialog->m_radio_3)) {
+		if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(context_dialog->m_radio_3))) {
+			context_dialog->SetCurrentVoiceParamsTo(2);
+			context_dialog->m_current_voice = 2;
+		}
+		else {
+			context_dialog->GetCurrentVoiceParamsFrom(2, context_dialog->m_volume, context_dialog->m_pan,
+							context_dialog->m_midi_program_number, &(context_dialog->m_config_changed));
+		}
+	}
+	else if (togglebutton == GTK_TOGGLE_BUTTON(context_dialog->m_radio_4)) {
+		if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(context_dialog->m_radio_4))) {
+			context_dialog->SetCurrentVoiceParamsTo(3);
+			context_dialog->m_current_voice = 3;
+		}
+		else {
+			context_dialog->GetCurrentVoiceParamsFrom(3, context_dialog->m_volume, context_dialog->m_pan,
+							context_dialog->m_midi_program_number, &(context_dialog->m_config_changed));
+		}
+	}
+}
