@@ -31,20 +31,27 @@
 class NedTextDialog {
 	public:
 		NedTextDialog(GtkWindow *parent, char *text, int anchor, unsigned int tempo, bool enable_tempo, unsigned int volume, bool enable_volume,
-			unsigned short segno_sign, const char *font_family, double font_size, PangoStyle font_slant, PangoWeight font_weight);
+			unsigned short segno_sign, int midi_program, int channel, bool enable_channel, const char *font_family, double font_size, PangoStyle font_slant, PangoWeight font_weight);
 		void getValues(bool *state, char **text, int *anchor, unsigned int *tempo, bool *enable_tempo, unsigned int *volume, bool *enable_volume, 
-			unsigned short *segno_sign, const char **font_family, double *font_size, PangoStyle *font_slant, PangoWeight *font_weight);
+			unsigned short *segno_sign, int *midi_program, unsigned int *channel, bool *enable_channel, const char **font_family, double *font_size, PangoStyle *font_slant, PangoWeight *font_weight);
 	private:
 		static void CloseCallBack(GtkDialog *dialog, gint result, gpointer data);
 		static void FontCallBack(GtkButton *button, gpointer data);
 		static void TempoToggled(GtkToggleButton *button, gpointer data);
 		static void VolumeToggled(GtkToggleButton *button, gpointer data);
+		static void InstrToggled(GtkToggleButton *button, gpointer data);
+		static void ChannelToggled(GtkToggleButton *button, gpointer data);
+		static gboolean OnScroll(GtkWidget *widget, GdkEventScroll *event, gpointer data);
+		void retrieveFontParams();
 		GtkWidget *m_text_widget;
 		GtkWidget *m_font_button;
 		GtkWidget *m_tempo_spin_box;
 		GtkWidget *m_volume_spin_box;
+		GtkWidget *m_channel_spin_box;
 		GtkWidget *m_check_tempo;
 		GtkWidget *m_check_volume;
+		GtkWidget *m_check_instr;
+		GtkWidget *m_check_channel;
 		GtkWidget *m_anchor_left;
 		GtkWidget *m_anchor_mid;
 		GtkWidget *m_anchor_right;
@@ -55,10 +62,17 @@ class NedTextDialog {
 		GtkWidget *m_d_capo_al_coda;
 		GtkWidget *m_fine;
 		GtkWidget *m_al_coda;
+		GtkWidget *m_instrument_list;
+		GtkObject *m_instrument_adjustment;
+		GtkWidget *m_main_h_box;
+		GtkWidget *m_instrument_scroll;
 		unsigned int m_tempo;
 		unsigned int m_volume;
 		bool m_enable_tempo, m_enable_volume;
 		unsigned short m_segno_sign;
+		int m_midi_program_number;
+		int m_channel;
+		bool m_enable_channel;
 		int m_anchor;
 		char *m_text;
 		const char *m_font_family;
@@ -66,6 +80,10 @@ class NedTextDialog {
 		PangoStyle m_font_slant;
 		PangoWeight m_font_weight;
 		bool m_state;
+		static const char *m_static_font_family;
+		static double m_static_font_size;
+		static PangoStyle m_static_font_slant;
+		static PangoWeight m_static_font_weight;
 };
 
 #endif /* TEXT_DIALOG_H */
