@@ -57,13 +57,16 @@ struct staff_context_str {
 	int m_key_signature_number;
 	int m_clef_number;
 	int m_clef_octave_shift;
-	unsigned char m_midi_volume;
-	unsigned char m_current_midi_volume;
-	unsigned char m_midi_pan;
+	bool m_different_voices;
+	struct {
+		unsigned char m_midi_volume;
+		unsigned char m_midi_pan;
+		unsigned char m_midi_program;
+		unsigned char m_current_midi_program;
+	} voices[VOICE_COUNT];
+	char m_play_transposed;
 	unsigned char m_midi_chorus;
 	unsigned char m_midi_reverb;
-	char m_play_transposed;
-	unsigned char m_midi_program;
 	unsigned char m_midi_channel;
 	unsigned char m_effective_channel;
 	unsigned int m_flags;
@@ -73,10 +76,6 @@ struct staff_context_str {
 #define BRACKET_END   (1 << 3)
 #define CON_BAR_START (1 << 4)
 #define CON_BAR_END   (1 << 5)
-	unsigned long long m_crescendo_midi_start;
-	unsigned long long m_crescendo_midi_end;
-	unsigned char m_crescendo_start_volume;
-	unsigned char m_crescendo_end_volume;
 	int m_pitch_offs;
 	bool m_muted;
 
@@ -119,6 +118,7 @@ class SpecialMeasure {
 		unsigned short numerator, denominator;
 		unsigned int symbol;
 		bool isNear(double x, double y);
+		static int compare_spec_measures(const SpecialMeasure *a, const SpecialMeasure *b);
 };
 
 #define NO_LYRICS -1 
@@ -203,6 +203,7 @@ class NedMainWindow {
 		void changeNoteHead();
 		void changeTieState();
 		void changeNoteState();
+		void changeTremolo();
 		int getNumerator(int meas_num);
 		int getDenominator(int meas_num);
 		unsigned int getTimesigSymbol(int meas_num);
@@ -223,20 +224,19 @@ class NedMainWindow {
 		void updatePageCounter();
 		void renumberPages();
 		int getStaffCount() {return m_staff_count;}
+		void do_import_midi(bool use_id_ed);
 		void do_remove_last_staff();
 		bool do_import_from_other(NedImporter *im);
 		void do_lily_export(FILE *fp, bool with_break, bool *selected_staves, bool *kept_beams);
 		void do_transpose(int pitchdist, bool *staff_list, GList *selected_group);
 		void shiftNotes(unsigned long long start_time, int linedist, NedPage *start_page, NedSystem *start_system, int staff_number);
-		void ProgramChannels();
 		int setChannelFromCursor();
 		void deleteStaff(int staff_number);
 		void shiftStaff(int staff_number, int position);
 		void restoreStaff(int staff_number, staff_context_str *staff_context);
-		unsigned int determineVolume(NedChordOrRest *element, int staff_nr);
 		double determineTempoInverse(NedChordOrRest *element);
 		void setInternalPitches();
-		void adjustAccidentals();
+		void adjustAccidentals(int staff_nr);
 		void insertNoteWithPitch(int num_midi_input_notes, int *midi_input_chord);
 		GtkWidget *getDrawingArea() {return m_drawing_area;}
 		GtkWidget *getWindow() {return m_main_window;}
@@ -283,7 +283,7 @@ class NedMainWindow {
 		bool conflictWithOhterSpecMeasure(int measnum, int dir);
 		void testAlternativeState(NedCommandList *command_list, NedMeasure *measure, bool alter1, bool alter2);
 		void checkForElementsToSplit(NedCommandList *command_list);
-		bool import_from_other(NedImporter *im, char* filename);
+		bool import_from_other(NedImporter *im, char* filename, bool execute_import);
 		void deleteEmptyPagesAtEnd();
 		void deletePagesWithoutSystem();
 		void removeEmptyInteriourPages();
@@ -393,6 +393,7 @@ class NedMainWindow {
 		static void print_file(GtkWidget *widget, void *data);
 		static void import_musicxml(GtkWidget *widget, void *data);
 		static void import_midi(GtkWidget *widget, void *data);
+		static void import_midi2(GtkWidget *widget, void *data);
 		static void export_midi(GtkWidget *widget, void *data);
 		static void export_lily(GtkWidget *widget, void *data);
 		static void insert_tuplet(GtkWidget *widget, void *data);
