@@ -69,11 +69,18 @@
 #define XML_STATE_TYPE 38
 #define XML_STATE_CLEF_OCTAVE_CHANGE 39
 #define XML_STATE_ARTICULATIONS 40
+#define XML_STATE_VOLUME 41
+#define XML_STATE_DIRECTION 42
+#define XML_STATE_DIR_TYPE 43
+#define XML_STATE_METRONOME 44
+#define XML_STATE_BEAT_UNIT 45
+#define XML_STATE_PER_MINUTE 46
 
 #define XML_TIE_TYPE_NONE 0
 #define XML_TIE_TYPE_START 1
 #define XML_TIE_TYPE_STOP 2
 
+#define CHOOSE_HEAD(line) ((im->m_current_part->midi_channel == 9) ? (((line + OFFSET_TO_BE_USED_DURING_MIDI_IMPORT) % COUNT_OF_DRUMS_TO_CHOOSE_DURING_MIDI_IMPORT) + 1) : NORMAL_NOTE ) 
 
 bool NedMusicXMLImport::import() {
 #define BUFLEN 4096
@@ -371,12 +378,30 @@ void NedMusicXMLImport::startTag (GMarkupParseContext *context,
 	else if (!strcmp(element_name, "midi-instrument")) {
 		im->push_state(XML_STATE_MIDI_INSTRUMENT);
 	}
+	else if (!strcmp(element_name, "direction")) {
+		im->push_state(XML_STATE_DIRECTION);
+	}
+	else if (!strcmp(element_name, "direction-type")) {
+		im->push_state(XML_STATE_DIR_TYPE);
+	}
+	else if (!strcmp(element_name, "metronome")) {
+		im->push_state(XML_STATE_METRONOME);
+	}
+	else if (!strcmp(element_name, "beat-unit")) {
+		im->push_state(XML_STATE_BEAT_UNIT );
+	}
+	else if (!strcmp(element_name, "per-minute")) {
+		im->push_state(XML_STATE_PER_MINUTE );
+	}
 	else if (!strcmp(element_name, "midi-program")) {
 		im->push_state(XML_STATE_MIDI_PGM);
 	}
 	else if (!strcmp(element_name, "midi-channel")) {
 		im->push_state(XML_STATE_MIDI_CHANNEL);
 	}
+	else if (!strcmp(element_name, "volume")) {
+		im->push_state(XML_STATE_VOLUME);
+	}
 	else if (!strcmp(element_name, "measure")) {
 		im->m_current_measure_number++;
 		im->push_state(XML_STATE_MEASURE);
@@ -401,10 +426,10 @@ void NedMusicXMLImport::startTag (GMarkupParseContext *context,
 			return;
 		}
 		if (!strcmp(buffer, "forward")) {
-			im->m_main_window->setSpecial(im->m_current_measure_number, REPEAT_OPEN, false, NULL);
+			im->m_main_window->setSpecial(im->m_current_measure_number, REPEAT_OPEN, false, NULL, false);
 		}
 		else if (!strcmp(buffer, "backward")) {
-			im->m_main_window->setSpecial(im->m_current_measure_number + 1, REPEAT_CLOSE, false, NULL);
+			im->m_main_window->setSpecial(im->m_current_measure_number + 1, REPEAT_CLOSE, false, NULL, false);
 		}
 	}
 	else if (!strcmp(element_name, "ending")) {
@@ -419,19 +444,19 @@ void NedMusicXMLImport::startTag (GMarkupParseContext *context,
 				return;
 			}
 			if (buffer[0] == '1' && buffer[1] == '\0') {
-				im->m_main_window->setSpecial(im->m_current_measure_number, REP1START, false, NULL);
+				im->m_main_window->setSpecial(im->m_current_measure_number, REP1START, false, NULL, false);
 			}
 			else if (buffer[0] == '2' && buffer[1] == '\0') {
-				im->m_main_window->setSpecial(im->m_current_measure_number, REP2START, false, NULL);
-				im->m_main_window->setSpecial(im->m_current_measure_number + 1, REP2END, false, NULL);
+				im->m_main_window->setSpecial(im->m_current_measure_number, REP2START, false, NULL, false);
+				im->m_main_window->setSpecial(im->m_current_measure_number + 1, REP2END, false, NULL, false);
 			}
 		}
 		else if (strcmp(buffer, "discontinue")) {
 			if (buffer[0] == '1' && buffer[1] == '\0') {
-				im->m_main_window->setSpecial(im->m_current_measure_number + 1, REP1END, false, NULL);
+				im->m_main_window->setSpecial(im->m_current_measure_number + 1, REP1END, false, NULL, false);
 			}
 			else if (buffer[0] == '2' && buffer[1] == '\0') {
-				im->m_main_window->setSpecial(im->m_current_measure_number + 1, REP2END, false, NULL);
+				im->m_main_window->setSpecial(im->m_current_measure_number + 1, REP2END, false, NULL, false);
 			}
 		}
 	}
@@ -535,6 +560,7 @@ void NedMusicXMLImport::endTag (GMarkupParseContext *context,
 	int line, length, dotcount;
 	unsigned int state;
 	int dummy;
+	unsigned int dummy2;
 	NedChordOrRest *chord_or_rest = NULL;
 	if (im->m_semantic_error) return;
 /*
@@ -582,7 +608,7 @@ void NedMusicXMLImport::endTag (GMarkupParseContext *context,
 			}
 			state = im->determineState();
 			if ((line = im->pitchToLine()) == UNKNOWN_LINE) return;
-			NedNote *note = new NedNote(im->m_current_part->last_chord[im->m_current_voice], line, NORMAL_NOTE, state);
+			NedNote *note = new NedNote(im->m_current_part->last_chord[im->m_current_voice], line, CHOOSE_HEAD(line), state);
 			im->m_current_part->last_chord[im->m_current_voice]->insertNoteAt(note);
 			if (im->m_tie_stop && im->m_current_voice >= 0 && im->m_current_voice < VOICE_COUNT) {
 				im->tie(im->m_current_part->last_chord[im->m_current_voice], note);
@@ -599,7 +625,7 @@ void NedMusicXMLImport::endTag (GMarkupParseContext *context,
 			state = im->determineState();
 			im->m_current_part->voices[im->m_current_voice] = g_list_append(
 			   im->m_current_part->voices[im->m_current_voice], 
-				im->m_current_part->last_chord[im->m_current_voice] = chord_or_rest = new NedChordOrRest(NULL, TYPE_NOTE, FALSE, line, dotcount, length, NORMAL_NOTE, state, 0));
+				im->m_current_part->last_chord[im->m_current_voice] = chord_or_rest = new NedChordOrRest(NULL, TYPE_CHORD, FALSE, line, dotcount, length, CHOOSE_HEAD(line), state, 0));
 				for (int i = 0; i < MAX_LYRICS_LINES; i++) {
 					chord_or_rest->setLyricsFromOldContext(im->m_main_window, i, im->m_lyrics[i]);
 				}
@@ -647,7 +673,7 @@ void NedMusicXMLImport::endTag (GMarkupParseContext *context,
 		im->determineTimeSignature();
 		if (!im->m_current_part->first_time_signature) {
 			im->m_main_window->setSpecialTimesig(im->m_current_measure_number, im->m_current_part->current_numerator, 
-				im->m_current_part->current_denominator, &dummy, &dummy); // friend !!
+				im->m_current_part->current_denominator, TIME_SYMBOL_NONE, &dummy, &dummy, &dummy2); // friend !!
 		}
 		im->m_current_part->first_time_signature = false;
 
@@ -689,7 +715,10 @@ void NedMusicXMLImport::endTag (GMarkupParseContext *context,
 			!strcmp(element_name, "barline") || !strcmp(element_name, "repeat") ||
 			!strcmp(element_name, "ending") || !strcmp(element_name, "lyric") ||
 			!strcmp(element_name, "text") || !strcmp(element_name, "type") ||
-			!strcmp(element_name, "clef-octave-change") || !strcmp(element_name, "articulations")) {
+			!strcmp(element_name, "clef-octave-change") || !strcmp(element_name, "articulations") ||
+			!strcmp(element_name, "volume") || !strcmp(element_name, "direction") ||
+			!strcmp(element_name, "direction-type") || !strcmp(element_name, "metronome") ||
+			!strcmp(element_name, "beat-unit") || !strcmp(element_name, "per-minute")) {
 		im->pop_state();
 	}
 }
@@ -702,6 +731,7 @@ void NedMusicXMLImport::internal_text(GMarkupParseContext *context,
 	NedMusicXMLImport *im = (NedMusicXMLImport *) user_data;
 	if (im->m_semantic_error) return;
 	char Str[DEFAULT_STRING_LENGTH];
+	int vol, tempo;
 /*
 	int i;
 	NedResource::DbgMsg(DBG_TESTING, "TEXT(0): %s, state = %d\n", text, im->getState()); 
@@ -804,6 +834,42 @@ void NedMusicXMLImport::internal_text(GMarkupParseContext *context,
 				      }
 				      im->m_current_part->midi_channel--;
 				      break;
+		case XML_STATE_VOLUME: if (!im->check_state(XML_STATE_VOLUME, XML_STATE_MIDI_INSTRUMENT, XML_STATE_SCORE_PART)) {
+					im->produceSementicErrorString(_("\"volume\" outside \"midi-instrument\""));
+					return;
+				      }
+				      if (im->m_partcount >= MAXPARTS) break;
+				      if (im->m_current_part == NULL) {
+					NedResource::Abort("im->m_current_part == NULL(44)");
+				      }
+				      if (sscanf(text, "%d", &vol) != 1) {
+					sprintf(Str, _("Bad midi volume number: %s"), text);
+					im->produceSementicErrorString(Str);
+				      }
+				      if (vol < 0 || vol > 100) {
+					sprintf(Str, _("Bad volume number: %d"), vol);
+					im->produceSementicErrorString(Str);
+				      }
+				       im->m_current_part->volume = (int) ((double) vol * 127.0 / 100.0);
+				      break;
+		case XML_STATE_PER_MINUTE : if (!im->check_state(XML_STATE_PER_MINUTE , XML_STATE_METRONOME, XML_STATE_DIR_TYPE)) {
+					im->produceSementicErrorString(_("\"per-minute\" outside \"XML_STATE_DIRECTION\""));
+					return;
+				      }
+				      if (im->m_partcount >= MAXPARTS) break;
+				      if (im->m_current_part == NULL) {
+					NedResource::Abort("im->m_current_part == NULL(45)");
+				      }
+				      if (sscanf(text, "%d", &tempo) != 1) {
+					sprintf(Str, _("Bad midi volume number: %s"), text);
+					im->produceSementicErrorString(Str);
+				      }
+				      if (tempo < 20 || vol > 300) {
+					sprintf(Str, _("Bad temo value: %d"), tempo);
+					im->produceSementicErrorString(Str);
+				      }
+				       im->m_tempo_inverse = 60000.0 / (double) tempo;
+				      break;
 		case XML_STATE_SIGN: if (!im->check_state(XML_STATE_SIGN, XML_STATE_CLEF, XML_STATE_ATTRIBUTES)) {
                                         im->produceSementicErrorString(_("\"sign\" outside \"clef\""));
                                         return;
@@ -1287,11 +1353,11 @@ void NedMusicXMLImport::tie(NedChordOrRest *chord, NedNote *note) {
 		produceSementicErrorString(_("no tie back possible"));
 		return;
 	}
-	if (((NedChordOrRest *) lptr1->data)->getType() != TYPE_NOTE) {
+	if (((NedChordOrRest *) lptr1->data)->getType() != TYPE_CHORD) {
 		produceSementicErrorString(_("predecessor of tied is not a chord"));
 		return;
 	}
 	if (!((NedChordOrRest *)lptr2->data)->tieBackward((NedChordOrRest *) lptr1->data, note)) {
-		produceSementicErrorString(_("no tie partner"));
+		//produceSementicErrorString(_("no tie partner"));
 	}
 }
