@@ -75,7 +75,7 @@ int NedNote::signToBase[][2] = {
 	{-6, 66}, {-5, 61}, {-4, 68}, {-3, 63}, {-2, 70 }, {-1, 65}, {0, 60}, {1, 67}, {2, 62}, {3, 69}, {4, 64}, {5, 71}, {6, 66}
 };
 
-int NedNote::diaDists[] = {-1, 0, 2, 4, 5, 7, 9, 11, 12, 14};
+int NedNote::diaDists[] = {-7, -5, -3, -1, 0, 2, 4, 5, 7, 9, 11, 12, 14, 16, 17, 19};
 
 NedNote::NedNote(NedChordOrRest *chord_or_rest, int line, int head, unsigned int status) :
 m_active(NOTE_ACTIVE_STATE_NONE), m_xpos(0.0), m_ypos(0.0), m_tmp_ypos(0.0), m_acc_dist(0.0), m_line(line), m_tmp_line(UNKNOWN_LINE), m_status(status), m_head(head), m_chord_or_rest(chord_or_rest),
@@ -247,22 +247,22 @@ void NedNote::draw(cairo_t *cr) {
 	if (m_chord_or_rest->getDotCount() > 0) {
 		cairo_new_path(cr);
 		if ((m_line % 2) == 0 && m_line >= 0 && m_line <= 8) {
-			cairo_arc(cr, X_POS_PAGE_REL(m_xpos + m_chord_or_rest->getXPosAndMicroshifts() + HEAD_THICK + DOT1POS), Y_POS_STAFF_REL(yy - LINE_DIST / 2.0),
+			cairo_arc(cr, X_POS_PAGE_REL(m_chord_or_rest->getXPos() + m_chord_or_rest->getBBox()->x + m_chord_or_rest->getBBox()->width_netto - 2 * DOT_RAD), Y_POS_STAFF_REL(yy - LINE_DIST / 2.0),
 				zoom_factor * DOT_RAD, 0, 2.0 * M_PI);
 		}
 		else {
-			cairo_arc(cr, X_POS_PAGE_REL(m_xpos + m_chord_or_rest->getXPosAndMicroshifts() + HEAD_THICK + DOT1POS), Y_POS_STAFF_REL(yy),
+			cairo_arc(cr, X_POS_PAGE_REL(m_chord_or_rest->getXPos() + m_chord_or_rest->getBBox()->x + m_chord_or_rest->getBBox()->width_netto - 2 * DOT_RAD), Y_POS_STAFF_REL(yy),
 				zoom_factor * DOT_RAD, 0, 2.0 * M_PI);
 		}
 		cairo_fill(cr);
 		if (m_chord_or_rest->getDotCount() > 1) {
 			cairo_new_path(cr);
 			if ((m_line % 2) == 0 && m_line >= 0 && m_line <= 8) {
-				cairo_arc(cr, X_POS_PAGE_REL(m_xpos + m_chord_or_rest->getXPosAndMicroshifts() + HEAD_THICK + DOT2POS), Y_POS_STAFF_REL(yy - LINE_DIST / 2.0),
+				cairo_arc(cr, X_POS_PAGE_REL(m_chord_or_rest->getXPos() + m_chord_or_rest->getBBox()->x + m_chord_or_rest->getBBox()->width_netto - 2 * DOT_RAD - (DOT2POS - DOT1POS)), Y_POS_STAFF_REL(yy - LINE_DIST / 2.0),
 					zoom_factor * DOT_RAD, 0, 2.0 * M_PI);
 			}
 			else {
-				cairo_arc(cr, X_POS_PAGE_REL(m_xpos + m_chord_or_rest->getXPosAndMicroshifts() + HEAD_THICK + DOT2POS), Y_POS_STAFF_REL(yy),
+				cairo_arc(cr, X_POS_PAGE_REL(m_chord_or_rest->getXPos() + m_chord_or_rest->getBBox()->x + m_chord_or_rest->getBBox()->width_netto - 2 * DOT_RAD - (DOT2POS - DOT1POS)), Y_POS_STAFF_REL(yy),
 					zoom_factor * DOT_RAD, 0, 2.0 * M_PI);
 			}
 			cairo_fill(cr);
@@ -321,7 +321,7 @@ bool NedNote::isFirstNote() {
 }
 
 
-int NedNote::findPitchUpDown(int key, int pitch, bool up) {
+int NedNote::findPitchUpDown(int key, int pitch, int dist) {
 	int base = 60;
 	int i;
 
@@ -335,14 +335,10 @@ int NedNote::findPitchUpDown(int key, int pitch, bool up) {
 	while (pitch < base) base -= 12;
 	for (i = 1; i < 9; i++) {
 		if (base + diaDists[i] == pitch) {
-			if (up) {
-				return base + diaDists[i+1];
-			}
-			return base + diaDists[i-1];
+			return base + diaDists[i+dist];
 		}
 	}
-	if (up) return pitch + 1;
-	return pitch - 1;
+	return pitch + dist;
 }
 
 bool NedNote::testForTiesToDelete(NedCommandList *command_list, unsigned int dir /* = BREAK_TIE_BACKWARD */, bool execute /* = false */) {
@@ -741,69 +737,133 @@ void NedNote::changeAccidental(NedCommandList *command_list, int preferred_offs,
 }
 
 
-void NedNote::prepareReplay(unsigned long long midi_start_time, int clef, int keysig, int octave_shift, int pitch_offs, 
+void NedNote::prepareReplay(unsigned long long midi_start_time, int clef, int keysig, int octave_shift, int pitch_offs, int voice_idx, 
 		 unsigned int grace_time, bool is_grace, bool is_stacc, unsigned short segno_sign) {
-	int pitch, pitch2, tt;
-	unsigned int i;
+	int pitches[4], tt;
+	unsigned int i, part_duration;
 	bool dummy;
 
-	pitch = getPitch(clef, keysig, octave_shift, pitch_offs, &dummy, true);
+	pitches[0] = getPitch(clef, keysig, octave_shift, pitch_offs, &dummy, true);
 	if (is_grace) {
-		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_effective_channel,
-			midi_start_time - grace_time, SND_SEQ_EVENT_NOTEON, pitch, getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_current_midi_volume, segno_sign, this);
-		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_effective_channel,
-			midi_start_time - grace_time + GRACE_DURATION, SND_SEQ_EVENT_NOTEOFF, pitch, 0, segno_sign, this);
+		NedResource::addMidiNoteEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_effective_channel,
+			voice_idx,
+			midi_start_time - grace_time, SND_SEQ_EVENT_NOTEON, pitches[0],
+			getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].voices[voice_idx].m_current_midi_program, segno_sign, this);
+		NedResource::addMidiNoteEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_effective_channel,
+			voice_idx,
+			midi_start_time - grace_time + GRACE_DURATION, SND_SEQ_EVENT_NOTEOFF, pitches[0], 0, segno_sign, this);
 	}
 	else if (is_stacc && m_chord_or_rest->getLength() > NOTE_32 && grace_time == 0) {
-		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_effective_channel,
-			midi_start_time, SND_SEQ_EVENT_NOTEON, pitch, getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_current_midi_volume, segno_sign, this);
-		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_effective_channel,
-			m_chord_or_rest->getMidiTime() + m_chord_or_rest->getDuration() / 2, SND_SEQ_EVENT_NOTEOFF, pitch, 0, segno_sign, this);
+		NedResource::addMidiNoteEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_effective_channel,
+			voice_idx,
+			midi_start_time, SND_SEQ_EVENT_NOTEON, pitches[0],
+			getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].voices[voice_idx].m_current_midi_program, segno_sign, this);
+		NedResource::addMidiNoteEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_effective_channel,
+			voice_idx,
+			m_chord_or_rest->getMidiTime() + m_chord_or_rest->getDuration() / 2, SND_SEQ_EVENT_NOTEOFF, pitches[0], 0, segno_sign, this);
 	}
 	else if (grace_time == 0 && (m_chord_or_rest->getStatus() & STAT_TRILL) != 0 && (m_chord_or_rest->getStatus() & (STAT_STACC | STAT_STACCATISSIMO)) == 0) {
-		pitch2 = findPitchUpDown(keysig, pitch, true);
+		pitches[1] = findPitchUpDown(keysig, pitches[0], 1);
 		for (tt = 0, i = 0; i < m_chord_or_rest->getDuration() / NOTE_32; i++, tt = 1 - tt) {
-			NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_effective_channel,
-			midi_start_time + i * NOTE_32, SND_SEQ_EVENT_NOTEON, tt ? pitch : pitch2, getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_current_midi_volume, segno_sign, this);
-			NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_effective_channel,
-			m_chord_or_rest->getMidiTime() + (i + 1) * NOTE_32, SND_SEQ_EVENT_NOTEOFF, tt ? pitch : pitch2, 0, segno_sign, this);
+			NedResource::addMidiNoteEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_effective_channel,
+			voice_idx,
+			midi_start_time + i * NOTE_32, SND_SEQ_EVENT_NOTEON, tt ? pitches[0] : pitches[1],
+			getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].voices[voice_idx].m_current_midi_program, segno_sign, this);
+			NedResource::addMidiNoteEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_effective_channel,
+			voice_idx,
+			m_chord_or_rest->getMidiTime() + (i + 1) * NOTE_32, SND_SEQ_EVENT_NOTEOFF, tt ? pitches[1] : pitches[0], 0, segno_sign, this);
+		}
+	}
+	else if (m_chord_or_rest->getDuration() >= NOTE_8 && grace_time == 0 && (m_chord_or_rest->getStatus() & STAT_TURN) != 0 && (m_chord_or_rest->getStatus() & (STAT_STACC | STAT_STACCATISSIMO)) == 0) {
+		pitches[1] =  pitches[0];
+		pitches[0] = findPitchUpDown(keysig, pitches[1],  1);
+		pitches[2] = findPitchUpDown(keysig, pitches[1],  -1);
+		pitches[3] = pitches[1];
+		part_duration = m_chord_or_rest->getDuration() / 4;
+		for (i = 0; i < 4; i++) {
+			NedResource::addMidiNoteEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_effective_channel,
+			voice_idx,
+			midi_start_time + i * part_duration, SND_SEQ_EVENT_NOTEON, pitches[i],
+			getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].voices[voice_idx].m_current_midi_program, segno_sign, this);
+			NedResource::addMidiNoteEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_effective_channel,
+			voice_idx,
+			m_chord_or_rest->getMidiTime() + (i + 1) * part_duration, SND_SEQ_EVENT_NOTEOFF, pitches[i], 0, segno_sign, this);
+		}
+	}
+	else if (m_chord_or_rest->getDuration() >= NOTE_8 && grace_time == 0 && (m_chord_or_rest->getStatus() & STAT_REV_TURN) != 0 && (m_chord_or_rest->getStatus() & (STAT_STACC | STAT_STACCATISSIMO)) == 0) {
+		pitches[1] =  pitches[0];
+		pitches[0] = findPitchUpDown(keysig, pitches[1],  -1);
+		pitches[2] = findPitchUpDown(keysig, pitches[1],  1);
+		pitches[3] = pitches[1];
+		part_duration = m_chord_or_rest->getDuration() / 4;
+		for (i = 0; i < 4; i++) {
+			NedResource::addMidiNoteEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_effective_channel,
+			voice_idx,
+			midi_start_time + i * part_duration, SND_SEQ_EVENT_NOTEON, pitches[i],
+			getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].voices[voice_idx].m_current_midi_program, segno_sign, this);
+			NedResource::addMidiNoteEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_effective_channel,
+			voice_idx,
+			m_chord_or_rest->getMidiTime() + (i + 1) * part_duration, SND_SEQ_EVENT_NOTEOFF, pitches[i], 0, segno_sign, this);
 		}
 	}
 	else if (grace_time == 0 && (m_chord_or_rest->getStatus() & STAT_PRALL) != 0 && (m_chord_or_rest->getStatus() & (STAT_STACC | STAT_STACCATISSIMO)) == 0) {
-		pitch2 = findPitchUpDown(keysig, pitch, true);
-		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_effective_channel,
-		midi_start_time,  SND_SEQ_EVENT_NOTEON, pitch, getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_current_midi_volume, segno_sign, this);
-		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_effective_channel,
-		m_chord_or_rest->getMidiTime() +  NOTE_32, SND_SEQ_EVENT_NOTEOFF, pitch, 0, segno_sign, this);
-		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_effective_channel,
-		midi_start_time + NOTE_32,  SND_SEQ_EVENT_NOTEON, pitch2, getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_current_midi_volume, segno_sign, this);
-		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_effective_channel,
-		m_chord_or_rest->getMidiTime() + 2 * NOTE_32, SND_SEQ_EVENT_NOTEOFF, pitch2, 0, segno_sign, this);
-		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_effective_channel,
-		midi_start_time + 2 * NOTE_32,  SND_SEQ_EVENT_NOTEON, pitch, getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_current_midi_volume, segno_sign, this);
-		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_effective_channel,
-		m_chord_or_rest->getMidiTime() +  m_chord_or_rest->getDuration(), SND_SEQ_EVENT_NOTEOFF, pitch, 0, segno_sign, this);
+		pitches[1] = findPitchUpDown(keysig, pitches[0], 1);
+		NedResource::addMidiNoteEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_effective_channel,
+			voice_idx,
+		midi_start_time,  SND_SEQ_EVENT_NOTEON, pitches[0],
+		getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].voices[voice_idx].m_current_midi_program, segno_sign, this);
+		NedResource::addMidiNoteEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_effective_channel,
+			voice_idx,
+		m_chord_or_rest->getMidiTime() +  NOTE_32, SND_SEQ_EVENT_NOTEOFF, pitches[0], 0, segno_sign, this);
+		NedResource::addMidiNoteEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_effective_channel,
+			voice_idx,
+		midi_start_time + NOTE_32,  SND_SEQ_EVENT_NOTEON, pitches[1],
+		getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].voices[voice_idx].m_current_midi_program,
+		segno_sign, this);
+		NedResource::addMidiNoteEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_effective_channel,
+			voice_idx,
+		m_chord_or_rest->getMidiTime() + 2 * NOTE_32, SND_SEQ_EVENT_NOTEOFF, pitches[1], 0, segno_sign, this);
+		NedResource::addMidiNoteEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_effective_channel,
+			voice_idx,
+		midi_start_time + 2 * NOTE_32,  SND_SEQ_EVENT_NOTEON, pitches[0],
+		getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].voices[voice_idx].m_current_midi_program, segno_sign, this);
+		NedResource::addMidiNoteEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_effective_channel,
+			voice_idx,
+		m_chord_or_rest->getMidiTime() +  m_chord_or_rest->getDuration(), SND_SEQ_EVENT_NOTEOFF, pitches[0], 0, segno_sign, this);
 	}
 	else if (grace_time == 0 && (m_chord_or_rest->getStatus() & STAT_MORDENT) != 0 && (m_chord_or_rest->getStatus() & (STAT_STACC | STAT_STACCATISSIMO)) == 0) {
-		pitch2 = findPitchUpDown(keysig, pitch, false);
-		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_effective_channel,
-		midi_start_time,  SND_SEQ_EVENT_NOTEON, pitch, getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_current_midi_volume, segno_sign, this);
-		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_effective_channel,
-		m_chord_or_rest->getMidiTime() +  NOTE_32, SND_SEQ_EVENT_NOTEOFF, pitch, 0, segno_sign, this);
-		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_effective_channel,
-		midi_start_time + NOTE_32,  SND_SEQ_EVENT_NOTEON, pitch2, getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_current_midi_volume, segno_sign, this);
-		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_effective_channel,
-		m_chord_or_rest->getMidiTime() + 2 * NOTE_32, SND_SEQ_EVENT_NOTEOFF, pitch2, 0, segno_sign, this);
-		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_effective_channel,
-		midi_start_time + 2 * NOTE_32,  SND_SEQ_EVENT_NOTEON, pitch, getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_current_midi_volume, segno_sign, this);
-		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_effective_channel,
-		m_chord_or_rest->getMidiTime() +  m_chord_or_rest->getDuration(), SND_SEQ_EVENT_NOTEOFF, pitch, 0, segno_sign, this);
+		pitches[1] = findPitchUpDown(keysig, pitches[0], -1);
+		NedResource::addMidiNoteEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_effective_channel,
+			voice_idx,
+		midi_start_time,  SND_SEQ_EVENT_NOTEON, pitches[0],
+		getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].voices[voice_idx].m_current_midi_program,
+		segno_sign, this);
+		NedResource::addMidiNoteEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_effective_channel,
+			voice_idx,
+		m_chord_or_rest->getMidiTime() +  NOTE_32, SND_SEQ_EVENT_NOTEOFF, pitches[0], 0, segno_sign, this);
+		NedResource::addMidiNoteEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_effective_channel,
+			voice_idx,
+		midi_start_time + NOTE_32,  SND_SEQ_EVENT_NOTEON, pitches[1],
+		getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].voices[voice_idx].m_current_midi_program, segno_sign, this);
+		NedResource::addMidiNoteEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_effective_channel,
+			voice_idx,
+		m_chord_or_rest->getMidiTime() + 2 * NOTE_32, SND_SEQ_EVENT_NOTEOFF, pitches[1], 0, segno_sign, this);
+		NedResource::addMidiNoteEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_effective_channel,
+			voice_idx,
+		midi_start_time + 2 * NOTE_32,  SND_SEQ_EVENT_NOTEON, pitches[0],
+		getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].voices[voice_idx].m_current_midi_program, segno_sign, this);
+		NedResource::addMidiNoteEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_effective_channel,
+			voice_idx,
+		m_chord_or_rest->getMidiTime() +  m_chord_or_rest->getDuration(), SND_SEQ_EVENT_NOTEOFF, pitches[0], 0, segno_sign, this);
 	}
 	else {
-		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_effective_channel,
-			midi_start_time, SND_SEQ_EVENT_NOTEON, pitch, getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_current_midi_volume, segno_sign, this);
-		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_effective_channel,
-			m_chord_or_rest->getMidiTime() + m_chord_or_rest->getDuration() - grace_time, SND_SEQ_EVENT_NOTEOFF, pitch, 0, segno_sign, this);
+		NedResource::addMidiNoteEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_effective_channel,
+			voice_idx,
+			midi_start_time, SND_SEQ_EVENT_NOTEON, pitches[0],
+			getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].voices[voice_idx].m_current_midi_program, segno_sign, this);
+		NedResource::addMidiNoteEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_effective_channel,
+			voice_idx,
+			m_chord_or_rest->getMidiTime() + m_chord_or_rest->getDuration() - grace_time, SND_SEQ_EVENT_NOTEOFF, pitches[0], 0, segno_sign, this);
 	}
 }
 
@@ -935,6 +995,7 @@ bool NedNote::testYShift(double y) {
 }
 
 void NedNote::setStatus(unsigned int status) {
+	int voice_idx;
 	int clef, keysig, octave_shift;
 	int pitch;
 	bool dummy;
@@ -942,13 +1003,20 @@ void NedNote::setStatus(unsigned int status) {
 	m_status = status;
 	context = &(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()]);
 	getStaff()->getCurrentClefAndKeysig(m_chord_or_rest->getMidiTime(), &clef, &keysig, &octave_shift);
+	if (getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_different_voices) {
+		voice_idx = m_chord_or_rest->getVoice()->getVoiceNumber();
+	}
+	else {
+		voice_idx = 0;
+	}
 	if (NedResource::m_midi_echo) {
 		pitch = getPitch(clef, keysig, octave_shift, 0, &dummy, true);
-		NedResource::playImmediately(context->m_midi_channel, context->m_midi_program, 1, &pitch, context->m_midi_volume);
+		NedResource::playImmediately(context->m_midi_channel, context->voices[voice_idx].m_midi_program, 1, &pitch, context->voices[voice_idx].m_midi_volume);
 	}
 }
 void NedNote::moveUpDown(int line) {
 	staff_context_str *context;
+	int voice_idx;
 	bool dummy;
 	int clef, keysig, octave_shift;
 	int pitch;
@@ -956,10 +1024,16 @@ void NedNote::moveUpDown(int line) {
 	m_line = line;
 	m_ypos = - m_line * LINE_DIST/2.0;
 	m_chord_or_rest->sortNotes();
+	if (getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_different_voices) {
+		voice_idx = m_chord_or_rest->getVoice()->getVoiceNumber();
+	}
+	else {
+		voice_idx = 0;
+	}
 	if (NedResource::m_midi_echo) {
 		context = &(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()]);
 		pitch = getPitch(clef, keysig, octave_shift, 0, &dummy, true);
-		NedResource::playImmediately(context->m_midi_channel, context->m_midi_program, 1, &pitch, context->m_midi_volume);
+		NedResource::playImmediately(context->m_midi_channel, context->voices[voice_idx].m_midi_program, 1, &pitch, context->voices[voice_idx].m_midi_volume);
 	}
 	m_tmp_line = UNKNOWN_LINE;
 }
