@@ -93,7 +93,7 @@ class NedNote {
 		void adjustAccidental(int clef, int keysig, int octave_shift);
 		void changeAccidental(NedCommandList *command_list, int preferred_offs, int clef, int keysig, int octave_shift);
 		void prepareReplay(unsigned long long midi_start_time, int clef, int keysig, 
-			 int octave_shift,  int pitch_offs, unsigned int grace_time, bool is_grace, bool is_stacc, unsigned short segno_sign);
+			 int octave_shift,  int pitch_offs, int voice_idx, unsigned int grace_time, bool is_grace, bool is_stacc, unsigned short segno_sign);
 		char m_active;
 		int getPitch(int clef, int keysig, int octave_shift, int pitch_offs, bool *flat, bool use_transpose);
 		void removeUnneededAccidental(int clef, int keysig, int octave_shift);
@@ -109,7 +109,7 @@ class NedNote {
 		bool isFirstNote();
 		void setTieOffset(double offs) {m_tie_offs = offs;}
 	private:
-		int findPitchUpDown(int key, int pitch, bool up);
+		int findPitchUpDown(int key, int pitch, int dist);
 		int getDrumPitch();
 		void exportLilyPondDrums(FILE *fp);
 		static pitch_descr lineToPitch[];
