@@ -286,17 +286,25 @@
 #define STAT_MORDENT   (1 << 15)
 #define STAT_OPEN      (1 << 16)
 #define STAT_STACCATISSIMO (1 << 17)
-#define STAT_FERMATA (1 << 18)
-#define STAT_ARPEGGIO (1 << 19)
+#define STAT_FERMATA   (1 << 18)
+#define STAT_ARPEGGIO  (1 << 19)
 #define STAT_BREAK_BEAM (1 << 20)
 #define STAT_FORCE_BEAM (1 << 21)
-#define CHORD_STAT_MASK (STAT_STACC  | STAT_PIZZ      | STAT_TENUTO    | \
-                         STAT_SFORZATO  | STAT_SFORZANDO | STAT_BOW_UP    | STAT_BOW_DOWN  | \
-                         STAT_PED_ON    | STAT_PED_OFF   | STAT_TRILL     | STAT_PRALL     | \
-                         STAT_MORDENT   | STAT_OPEN      | STAT_STACCATISSIMO | STAT_FERMATA | STAT_ARPEGGIO )
 #define STAT_USER_STEMDIR (1 << 22)
 #define STAT_USER_HIDDEN_REST  (1 << 23)
 #define STAT_USER_PLACED_REST  (1 << 24)
+#define STAT_TURN      (1 << 25)
+#define STAT_REV_TURN   (1 << 26)
+#define STAT_TREMOLO1   (0x1 << 27)
+#define STAT_TREMOLO2   (0x2 << 27)
+#define STAT_TREMOLO3   (0x3 << 27)
+#define STAT_TREMOLO4   (0x4 << 27)
+#define TREMOLO_MASK    (0x7 << 27)
+#define CHORD_STAT_MASK (STAT_STACC  | STAT_PIZZ      | STAT_TENUTO    | \
+                         STAT_SFORZATO  | STAT_SFORZANDO | STAT_BOW_UP    | STAT_BOW_DOWN  | \
+                         STAT_PED_ON    | STAT_PED_OFF   | STAT_TRILL     | STAT_PRALL     | \
+                         STAT_MORDENT   | STAT_OPEN      | STAT_STACCATISSIMO | STAT_FERMATA | STAT_ARPEGGIO | \
+			 STAT_TURN | STAT_REV_TURN | TREMOLO_MASK )
 
 #define VIRTUAL_NATURAL_OFFS -111
 
@@ -429,8 +437,6 @@
 #define TOLERANCE 0.05
 #define MIN_REST_Y_VOICE_DIST 0.1
 
-#define MIN_VOICE_ELEMS_DIST 0.04
-
 #define MAX_OVER_UNDER_LINES 12
 
 #define OFFS_ARRAY_SIZE 115
@@ -486,6 +492,11 @@ struct addr_ref_str {
 	struct addr_ref_str *next;
 };
 
+struct MidiListStruct {
+	double tempoinverse;
+	GList *notes;
+};
+
 struct MidiEventStruct {
 	unsigned short channel, segno_sign;
 	unsigned long long midi_time;
@@ -495,13 +506,20 @@ struct MidiEventStruct {
 #define PSEUDO_MASK (0xffff << 16)
 #define NOTE_MASK (0xffff)
 #define PSEUDO_EVENT_MEASURE (PSEUDO_FLAG | 0x1)
-#define PSEUDO_EVENT_TEMPO_CHANGE (PSEUDO2_FLAG | 0x2)
-#define PSEUDO_EVENT_KEY_CHANGE (PSEUDO2_FLAG | 0x4)
-#define PSEUDO_EVENT_PGM_CHANGE (PSEUDO2_FLAG | 0x8)
-	int data1, data2;
+#define PSEUDO_EVENT_VOL_CHANGE (PSEUDO_FLAG | 0x2)
+#define PSEUDO_EVENT_RIT_START (PSEUDO2_FLAG | 0x1)
+#define PSEUDO_EVENT_RIT_STOP (PSEUDO2_FLAG | 0x2)
+#define PSEUDO_EVENT_CRES_START (PSEUDO2_FLAG | 0x4)
+#define PSEUDO_EVENT_CRES_STOP (PSEUDO2_FLAG | 0x8)
+#define PSEUDO_EVENT_TEMPO_CHANGE (PSEUDO2_FLAG | 0x10)
+#define PSEUDO_EVENT_KEY_CHANGE (PSEUDO2_FLAG | 0x20)
+	unsigned char data2;
+	unsigned char pgm; // for notes;
+	unsigned char voice;
+	double tempoinverse;
+	int data1;
 	NedNote *note;
 	NedMeasure *measure;
-	double tempoinverse;
 	bool repetition;
 };
 
@@ -512,7 +530,21 @@ struct paper_info_struct {
 
 class NedBbox {
 	public: 
-		double x, y, width, height;
+		double x, y, width, width_netto /* width without lyrics */, height;
+};
+
+struct crescendo_info {
+	unsigned int symbol;
+	GList *listptr;
+	unsigned long long midi_time;
+	int volume;
+};
+
+struct accel_info {
+	unsigned int symbol;
+	GList *listptr;
+	unsigned long long midi_time;
+	double tempo_inverse;
 };
 
 class NedResource {
@@ -629,6 +661,11 @@ class NedResource {
 		static GdkPixbuf *m_isolate_icon;
 		static GdkPixbuf *m_flipstem_icon;
 
+		static GdkPixbuf *m_tremolo_icon1;
+		static GdkPixbuf *m_tremolo_icon2;
+		static GdkPixbuf *m_tremolo_icon3;
+		static GdkPixbuf *m_tremolo_icon4;
+
 		static GdkPixbuf *m_brace_icon;
 		static GdkPixbuf *m_bracket_icon;
 
@@ -640,6 +677,8 @@ class NedResource {
 		static GdkPixbuf *m_mordent_icon;
 		static GdkPixbuf *m_fermata_icon;
 		static GdkPixbuf *m_arpeggio_icon;
+		static GdkPixbuf *m_turn_icon;
+		static GdkPixbuf *m_revturn_icon;
 
 		static GdkPixbuf *m_comm_icon;
 		static GdkPixbuf *m_cut_icon;
@@ -688,14 +727,12 @@ class NedResource {
 		static void startMidiInput();
 		static void stopMidiInput();
 		static void close_sequ();
-		static void MidiProgramChange(int channel, int pgm);
 		static void MidiCtrl(unsigned int ctrl_param, int channel, int val);
-		static void NoteOn(int channel, int pitch, int vol);
+		static void NoteOn(int channel, int pitch, int pgm, int vol);
 		static void NoteOff(int channel, int pitch);
 		static void SeqReset();
 
-		static void setAccelRange(unsigned int offset, unsigned long long duration, double tempodiff);
-		static void addMidiEvent(unsigned short channel, unsigned int offset, unsigned int type, int data1, int data2,
+		static void addMidiNoteEvent(unsigned short channel, unsigned char voice, unsigned int offset, unsigned int type, int data1, int pgm, 
 			unsigned short segno_sign, NedNote *note);
 		static void addMidiCtrlEvent(unsigned short channel, unsigned int offset, unsigned int ctrl_param, int val);
 		static void addTempoChange(unsigned int offset, double tempoinverse);
@@ -704,7 +741,8 @@ class NedResource {
 		static void addMeasureEntry(NedMeasure *measure);
 		static void addTempoChange(double tempoinverse);
 		static void addFermata(unsigned int offset);
-		static void addPgmChange(unsigned int offset, int chn, int pgm);
+		static void addPseudoEvent2(unsigned int offset, int event, int chn, int data);
+		static void setStartVolume(unsigned char chn, int voice_idx, unsigned char vol);
 		static void correctFermataTempo();
 		static void handleSegnos();
 		static void prepareMidiEventList(double tempo_inverse);
@@ -771,7 +809,6 @@ class NedResource {
 		static int determineLastLine(int treble_line, int clef);
 		static const char *getLilyPondClefName(int clef_number);
 		static const char *getLilyPondKeySigName(int keysig_number);
-		static void makeLilyString(char *ori, char *buffer);
 		static void writeLilyFreeChord(FILE *fp, NedChordName *chordname, unsigned int length, unsigned int duration, int dotcount, unsigned int *midi_len);
 		static unsigned int getCurrentLength();
 		static int getCurrentNoteHead();
@@ -801,6 +838,7 @@ class NedResource {
 		static gint m_toolboxX, m_toolboxY;
 	private:
 		static void printMidiEvent(MidiEventStruct *ev);
+		static void handleVolAndTempoChanges(double tempo_inverse);
 		static void removeAllPseudosEvents();
 		static GList *findListAtMeasure(int meas_number, NedMeasure **measure);
 		static void addTimeOffsetAfter(GList *end_ptr, unsigned long long time_diff);
@@ -824,7 +862,11 @@ class NedResource {
 		static snd_seq_addr_t m_alsa_out_dest_addr;
 		static snd_seq_addr_t m_alsa_in_src_addr;
 		static unsigned long long m_system_start_time;
-		static bool m_channel_list[16];
+		static int m_channel_pgms[16];
+		static int m_channel_volume[16][VOICE_COUNT];
+		static struct crescendo_info m_last_crescendo_info[16];
+		static double m_channel_volume_ratio[16][VOICE_COUNT];
+		static struct accel_info m_last_ritardando;
 		static GList *m_addr_rel_list;
 		static void write_config();
 		static void read_config();
@@ -861,10 +903,6 @@ class NedResource {
 		static snd_seq_event_t m_last_imm_notes[MAX_MIDI_INPUT_CHORD];
 		static GList *m_temp_file_names;
 		static double m_tempo_inverse;
-		static unsigned long long m_accel_midi_start;
-		static unsigned long long m_accel_midi_end;
-		static double m_accel_start_tempoinverse;
-		static double m_accel_end_tempoinverse;
 		static GList *m_fermata_list;
 		static bool m_ds_al_coda_available, m_ds_al_fine_available;
 		static bool m_d_capo_al_fine_available, m_d_capo_al_coda_available;
