@@ -157,10 +157,6 @@ void NedStaff::setAndUpdateClefTypeAndKeySig(int *clef_and_key_array, bool first
 	}
 }
 
-void NedStaff::determineVolume(unsigned int *vol, NedChordOrRest *element, int staff_nr) {
-	m_voices[0]->determineVolume(vol, element, staff_nr);
-}
-
 void NedStaff::determineTempoInverse(NedChordOrRest *element, unsigned long long till, double *tempoinverse, bool *found) {
 	int i;
 
@@ -169,6 +165,7 @@ void NedStaff::determineTempoInverse(NedChordOrRest *element, unsigned long long
 	}
 }
 
+
 GList *NedStaff::getFirstChordOrRest(int voice_nr, int lyrics_line, bool lyrics_required, bool note_required) {
 	if (voice_nr < 0 || voice_nr >= VOICE_COUNT) {
 		NedResource::Abort("NedStaff::getFirstChordOrRest");
@@ -1011,18 +1008,6 @@ void NedStaff::getCurrentClefAndKeysig(unsigned long long miditime, int *clef, i
 	}
 }
 
-int NedStaff::computeVolume(unsigned long long midi_time) {
-	GList *lptr;
-	VolumeChange *vol_change;
-	int volume = getMainWindow()->m_staff_contexts[m_staff_number].m_current_midi_volume;
-
-	for (lptr = g_list_first(m_volume_changes); lptr; lptr = g_list_next(lptr)) {
-		vol_change = (VolumeChange *) lptr->data;
-		if (vol_change->m_midi_time > midi_time) return volume;
-		volume = vol_change->m_volume;
-	}
-	return volume;
-}
 
 void NedStaff::prepareMutedReplay() {
 	int i;
@@ -1041,17 +1026,11 @@ void NedStaff::resetActiveFlags() {
 }
 
 void NedStaff::prepareReplay(bool with_keysig) {
-	int i;
+	int i, j;
 	GList *lptr;
 
-	m_volume_changes = NULL;
-
 	for (i = 0; i < VOICE_COUNT; i++) {
-		m_voices[i]->collectVolumeChanges(&m_volume_changes);
-	}
-
-	for (i = 0; i < VOICE_COUNT; i++) {
-		m_voices[i]->prepareReplay();
+		m_voices[i]->prepareReplay(getMainWindow()->m_staff_contexts[m_staff_number].m_different_voices ? i : 0);
 	}
 
 	if (with_keysig) {
@@ -1063,18 +1042,6 @@ void NedStaff::prepareReplay(bool with_keysig) {
 		}
 	}
 
-	if (m_volume_changes != NULL) {
-		if ((lptr = g_list_last(m_volume_changes)) != NULL) {
-			getMainWindow()->m_staff_contexts[m_staff_number].m_current_midi_volume =
-				((VolumeChange *) lptr->data)->m_volume;
-		}
-
-		for (lptr = g_list_first(m_volume_changes); lptr; lptr = g_list_next(lptr)) {
-			delete (VolumeChange *) lptr->data;
-		}
-		g_list_free(m_volume_changes);
-		m_volume_changes = NULL;
-	}
 }
 
 void NedStaff::findAccidentals(char offs_array[115], NedMeasure *meas_info, unsigned long long  midi_time, bool including) {
