@@ -584,12 +584,13 @@ void NedSystem::setInternalPitches() {
 	}
 }
 
-void NedSystem::adjustAccidentals() {
+void NedSystem::adjustAccidentals(int staff_nr) {
 	GList *lptr;
 
-	for (lptr = g_list_first(m_staffs); lptr; lptr = g_list_next(lptr)) {
-		((NedStaff *) lptr->data)->adjustAccidentals();
+	if ((lptr = g_list_nth(m_staffs, staff_nr)) == NULL) {
+		NedResource::Abort("NedSystem::adjustAccidentals");
 	}
+	((NedStaff *) lptr->data)->adjustAccidentals();
 }
 
 void NedSystem::changeAccidentals(NedCommandList *command_list, int preferred_offs, bool *staff_list, GList *selected_group) {
@@ -636,10 +637,18 @@ bool NedSystem::shiftNotes(unsigned long long start_time, int linedist, int staf
 	return ((NedStaff *) lptr->data)->shiftNotes(start_time, linedist);
 }
 
-void NedSystem::removeUnneededAccidentals() {
+void NedSystem::removeUnneededAccidentals(int staff_nr) {
 	GList *lptr;
 
-	for (lptr = g_list_first(m_staffs); lptr; lptr = g_list_next(lptr)) {
+	if (staff_nr < 0) {
+		for (lptr = g_list_first(m_staffs); lptr; lptr = g_list_next(lptr)) {
+			((NedStaff *) lptr->data)->removeUnneededAccidentals();
+		}
+	}
+	else {
+		if ((lptr = g_list_nth(m_staffs, staff_nr)) == NULL) {
+			NedResource::Abort("NedSystem::removeUnneededAccidentals");
+		}
 		((NedStaff *) lptr->data)->removeUnneededAccidentals();
 	}
 }
@@ -932,11 +941,7 @@ bool NedSystem::reposit(int measure_number, NedCommandList *command_list /* = NU
 	//do_reposit(use_upbeat);
 	m_check_line_compression = true;
 
-	if (m_extra_space < MINIMUM_EXTRA_SPACE) {
-	
-	
-	   if (m_measure_count > 1) {
-	//if (m_extra_space < MINIMUM_EXTRA_SPACE && m_measure_count > 1) {
+	if (m_extra_space < MINIMUM_EXTRA_SPACE && m_measure_count > 1) {
 		//printf("pag: %d sys: %d: m_extra_space(%f) < MINIMUM_EXTRA_SPACE (%f)reposit\n", m_page->getPageNumber(), m_system_number, m_extra_space, MINIMUM_EXTRA_SPACE); fflush(stdout);
 		m_check_line_compression = FALSE;
 		//critical_measure = m_special_measure_at_end ? m_measure_count-2 : m_measure_count-1;
@@ -974,7 +979,6 @@ bool NedSystem::reposit(int measure_number, NedCommandList *command_list /* = NU
 		command_list->addCommand(command);
 		command->execute();
 	}
-	}
 	if (m_check_line_compression && m_extra_space > MAXIMUM_EXTRA_SPACE) {
 		//printf("pag: %d sys: %d : m_extra_space(%f) > MAXIMUM_EXTRA_SPACE (%f)reposit\n", m_page->getPageNumber(), m_system_number, m_extra_space, MAXIMUM_EXTRA_SPACE); fflush(stdout);
 		if ((other_system = m_page->getNextSystem(this)) != NULL) {
@@ -1169,16 +1173,6 @@ void NedSystem::setAndUpdateClefTypeAndKeySig(int *clef_and_key_array, double in
 				  // do not update m_system_start correctly
 }
 
-void NedSystem::determineVolume(unsigned int *vol, NedChordOrRest *element, int staff_nr) {
-	GList *lptr;
-
-	if ((lptr = g_list_nth(m_staffs, staff_nr)) == NULL) {
-		NedResource::Abort("NedSystem::determineVolume");
-	}
-
-	((NedStaff *) lptr->data)->determineVolume(vol, element, staff_nr);
-}
-
 void NedSystem::determineTempoInverse(NedChordOrRest *element, unsigned long long till, double *tempoinverse, bool *found) {
 	GList *lptr;
 	for (lptr = g_list_first(m_staffs); lptr; lptr = g_list_next(lptr)) {
@@ -1289,14 +1283,18 @@ void NedSystem::checkForElementsToSplit(NedCommandList *command_list, int *measu
 	int i;
 	unsigned int meas_duration;
 	unsigned long long midi_time = 0;
+	unsigned long long end_time = getSystemEndTime();
 
-	for (i = 0; i <  m_measure_count; i++) {
+	for (i = 0; midi_time < end_time && i < MAX_MEASURES; i++) {
 		m_measures[i].midi_start = midi_time;
 		meas_duration = getMainWindow()->getMeasureDuration(*measure_number);
 		m_measures[i].setMeasureNumber((*measure_number)++, getMainWindow()->m_special_measures /* friend !! */, true);
 		midi_time += meas_duration;
 		m_measures[i].midi_end = midi_time;
 	}
+
+	m_measure_count = i; // otherwise " NedVoice::checkForElementsToSplit" crashes, because it cannot find the appropriate measure
+                             // if the measure count increases/decreases
 	for (lptr = g_list_first(m_staffs); lptr; lptr = g_list_next(lptr)) {
 		((NedStaff *) lptr->data)->checkForElementsToSplit(command_list);
 	}
@@ -1385,13 +1383,13 @@ void NedSystem::do_reposit(bool use_upbeat) {
 	for (i = 1; i < m_measure_count+m_special_measure_at_end; i++) {
 		measure_has_only_staff_members = true;
 		current_position = m_measures[i].computeAndSetPositions(current_position, &num_elements, &extra_space_for_accidentals, &measure_has_only_staff_members, false);
-			if (m_measures[i].barRequiresHideFollowing()) {
-				m_endMeasure = &(m_measures[i]);
-			}
-			m_measures[i-1].end = m_measures[i].start;
-			if (m_measures[i].getSpecial() & (START_TYPE_MASK | END_TYPE_MASK)) {
-				m_has_repeat_lines = true;
-			}
+		if (m_measures[i].barRequiresHideFollowing()) {
+			m_endMeasure = &(m_measures[i]);
+		}
+		m_measures[i-1].end = m_measures[i].start;
+		if (m_measures[i].getSpecial() & (START_TYPE_MASK | END_TYPE_MASK)) {
+			m_has_repeat_lines = true;
+		}
 	}
 	m_measures[i-1].end = m_width - RIGHT_DIST;
 	if (m_measures[i].getSpecial() & (START_TYPE_MASK | END_TYPE_MASK)) {
@@ -1698,11 +1696,11 @@ void NedSystem::detectVoices(int staff_nr, unsigned int *voice_mask, NedSystem *
 			*last_system = this;
 			*end_time = e_time;
 		}
-		else if ((*last_system)->m_system_number < m_system_number) {
+		else if ((*last_system)->m_page->getPageNumber() == m_page->getPageNumber() && (*last_system)->m_system_number < m_system_number) {
 			*last_system = this;
 			*end_time = e_time;
 		}
-		else if (*last_system == this) {
+		else if ((*last_system)->m_page->getPageNumber() == m_page->getPageNumber() && (*last_system)->m_system_number == m_system_number && *last_system == this) {
 			if (e_time > *end_time) {
 				*end_time = e_time;
 			}
